@@ -1,36 +1,46 @@
 /*
 ################################################################################
-# 
-# Copyright(c) Realtek Semiconductor Corp. All rights reserved.
-# 
-# This program is free software; you can redistribute it and/or modify it 
-# under the terms of the GNU General Public License as published by the Free 
-# Software Foundation; either version 2 of the License, or (at your option) 
+#
+# r8168 is the Linux device driver released for RealTek RTL8168B/8111B,
+# RTL8168C/8111C, RTL8168CP/8111CP, RTL8168D/8111D, and RTL8168DP/8111DP, and
+# RTK8168E/8111E Gigabit Ethernet controllers with PCI-Express interface.
+#
+# Copyright(c) 2011 Realtek Semiconductor Corp. All rights reserved.
+#
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the Free
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
-# 
-# This program is distributed in the hope that it will be useful, but WITHOUT 
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 # more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc., 59 
-# Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-# 
-# The full GNU General Public License is included in this distribution in the
-# file called LICENSE.
-# 
+# this program; if not, see <http://www.gnu.org/licenses/>.
+#
+# Author:
+# Realtek NIC software team <nicfae@realtek.com>
+# No. 2, Innovation Road II, Hsinchu Science Park, Hsinchu 300, Taiwan
+#
 ################################################################################
 */
 
+/*
+ * This product is covered by one or more of the following patents:
+ * US5,307,459, US5,434,872, US5,732,094, US6,570,884, US6,115,776, and US6,327,625.
+ */
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
 #define CHECKSUM_PARTIAL CHECKSUM_HW
 #endif
 
-#ifndef IRQ_HANDLED
-#define irqreturn_t void
-#define IRQ_HANDLED
-#define IRQ_NONE
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+ #define irqreturn_t void
+ #define IRQ_HANDLED	1
+ #define IRQ_NONE	0
+ #define IRQ_RETVAL(x)
 #endif
 
 #ifndef HAVE_FREE_NETDEV
@@ -41,12 +51,53 @@
 #define SET_NETDEV_DEV(net, pdev)
 #endif
 
+#ifndef SET_MODULE_OWNER
+#define SET_MODULE_OWNER(dev)
+#endif
+
+#ifndef SA_SHIRQ
+#define SA_SHIRQ IRQF_SHARED
+#endif
+
+#ifndef NETIF_F_GSO
+#define gso_size	tso_size
+#define gso_segs	tso_segs
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
+	#ifdef CONFIG_NET_POLL_CONTROLLER
+		#define RTL_NET_POLL_CONTROLLER dev->poll_controller=rtl8168_netpoll
+	#else
+		#define RTL_NET_POLL_CONTROLLER
+	#endif
+
+	#ifdef CONFIG_R8168_VLAN
+		#define RTL_SET_VLAN dev->vlan_rx_register=rtl8168_vlan_rx_register
+	#else
+		#define RTL_SET_VLAN
+	#endif
+
+	#define RTL_NET_DEVICE_OPS(ops)	dev->open=rtl8168_open; \
+					dev->hard_start_xmit=rtl8168_start_xmit; \
+					dev->get_stats=rtl8168_get_stats; \
+					dev->stop=rtl8168_close; \
+					dev->tx_timeout=rtl8168_tx_timeout; \
+					dev->set_multicast_list=rtl8168_set_rx_mode; \
+					dev->change_mtu=rtl8168_change_mtu; \
+					dev->set_mac_address=rtl8168_set_mac_address; \
+					dev->do_ioctl=rtl8168_do_ioctl; \
+					RTL_NET_POLL_CONTROLLER; \
+					RTL_SET_VLAN;
+#else
+	#define RTL_NET_DEVICE_OPS(ops)	dev->netdev_ops=&ops
+#endif
+
 //Due to the hardware design of RTL8111B, the low 32 bit address of receive
 //buffer must be 8-byte alignment.
-#undef NET_IP_ALIGN
-#define NET_IP_ALIGN 8
-
-#define CONFIG_R8168_NAPI 1
+#ifndef NET_IP_ALIGN
+#define NET_IP_ALIGN		2
+#endif
+#define RTK_RX_ALIGN		8
 
 #ifdef CONFIG_R8168_NAPI
 #define NAPI_SUFFIX	"-NAPI"
@@ -54,10 +105,15 @@
 #define NAPI_SUFFIX	""
 #endif
 
-#define RTL8168_VERSION "8.006.00" NAPI_SUFFIX
+#define RTL8168_VERSION "8.028.00" NAPI_SUFFIX
 #define MODULENAME "r8168"
 #define PFX MODULENAME ": "
 
+#define GPL_CLAIM "\
+r8168  Copyright (C) 2011  Realtek NIC software team <nicfae@realtek.com> \n \
+This program comes with ABSOLUTELY NO WARRANTY; for details, please see <http://www.gnu.org/licenses/>. \n \
+This is free software, and you are welcome to redistribute it under certain conditions; see <http://www.gnu.org/licenses/>. \n"
+
 #ifdef RTL8168_DEBUG
 #define assert(expr) \
         if(!(expr)) {					\
@@ -95,7 +151,7 @@
 #define MAC_PROTOCOL_LEN	2
 #endif
 
-#define RX_FIFO_THRESH	7	/* 7 means NO threshold, Rx buffer level before first PCI xfer. */
+#define Reserved2_data	7
 #define RX_DMA_BURST	7	/* Maximum PCI burst, '6' is 1024 */
 #define TX_DMA_BURST_unlimited	7
 #define TX_DMA_BURST_1024	6
@@ -105,7 +161,7 @@
 #define TX_DMA_BURST_64		2
 #define TX_DMA_BURST_32		1
 #define TX_DMA_BURST_16		0
-#define EarlyTxThld 	0x3F	/* 0x3F means NO early transmit */
+#define Reserved1_data 	0x3F
 #define RxPacketMaxSize	0x3FE8	/* 16K - 1 - ETH_HLEN - VLAN - CRC... */
 #define Jumbo_Frame_2k	(2 * 1024)
 #define Jumbo_Frame_3k	(3 * 1024)
@@ -120,14 +176,13 @@
 #define R8168_REGS_SIZE		256
 #define R8168_NAPI_WEIGHT	64
 
-//#define RX_BUF_SIZE	1536	/* Rx Buffer size */
-#define RX_BUF_SIZE	0x05F3	/* 0x05F3 = 1523 Rx Buffer size */
+#define RX_BUF_SIZE	0x05F3	/* 0x05F3 = 1522bye + 1 */
 #define R8168_TX_RING_BYTES	(NUM_TX_DESC * sizeof(struct TxDesc))
 #define R8168_RX_RING_BYTES	(NUM_RX_DESC * sizeof(struct RxDesc))
 
-#define RTL8168_TX_TIMEOUT	(6*HZ)
-#define RTL8168_PHY_TIMEOUT	(10*HZ)
-#define RTL8168_ESD_TIMEOUT	(2*HZ)
+#define RTL8168_TX_TIMEOUT	(6 * HZ)
+#define RTL8168_LINK_TIMEOUT	(1 * HZ)
+#define RTL8168_ESD_TIMEOUT	(2 * HZ)
 
 #define NUM_TX_DESC	1024	/* Number of Tx descriptor registers */
 #define NUM_RX_DESC	1024	/* Number of Rx descriptor registers */
@@ -190,6 +245,82 @@
 #define ADVERTISE_1000HALF	0x100
 #endif
 
+/*****************************************************************************/
+
+//#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,3)
+#if (( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,27) ) || \
+     (( LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) ) && \
+      ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,3) )))
+/* copied from linux kernel 2.6.20 include/linux/netdev.h */
+#define	NETDEV_ALIGN		32
+#define	NETDEV_ALIGN_CONST	(NETDEV_ALIGN - 1)
+
+static inline void *netdev_priv(struct net_device *dev)
+{
+	return (char *)dev + ((sizeof(struct net_device)
+					+ NETDEV_ALIGN_CONST)
+				& ~NETDEV_ALIGN_CONST);
+}
+#endif	//LINUX_VERSION_CODE < KERNEL_VERSION(2,6,3)
+
+/*****************************************************************************/
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,22)
+	#define	RTLDEV	tp
+#else
+	#define	RTLDEV	dev
+#endif	//LINUX_VERSION_CODE < KERNEL_VERSION(2,6,22)
+/*****************************************************************************/
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
+	typedef struct net_device *napi_ptr;
+	typedef int *napi_budget;
+
+	#define napi dev
+	#define RTL_NAPI_CONFIG(ndev, priv, function, weig)	ndev->poll=function;	\
+								ndev->weight=weig;
+	#define RTL_NAPI_QUOTA(budget, ndev)			min(*budget, ndev->quota)
+	#define RTL_GET_PRIV(stuct_ptr, priv_struct)		netdev_priv(stuct_ptr)
+	#define RTL_GET_NETDEV(priv_ptr)
+	#define RTL_RX_QUOTA(ndev, budget)			ndev->quota
+	#define RTL_NAPI_QUOTA_UPDATE(ndev, work_done, budget)	*budget -= work_done;	\
+								ndev->quota -= work_done;
+	#define RTL_NETIF_RX_COMPLETE(dev, napi)		netif_rx_complete(dev)
+	#define RTL_NETIF_RX_SCHEDULE_PREP(dev, napi)		netif_rx_schedule_prep(dev)
+	#define __RTL_NETIF_RX_SCHEDULE(dev, napi)		__netif_rx_schedule(dev)
+	#define RTL_NAPI_RETURN_VALUE				work_done >= work_to_do
+	#define RTL_NAPI_ENABLE(dev, napi)			netif_poll_enable(dev)
+	#define RTL_NAPI_DISABLE(dev, napi)			netif_poll_disable(dev)
+	#define DMA_BIT_MASK(value)				((1ULL << value) - 1)
+#else
+	typedef struct napi_struct *napi_ptr;
+	typedef int napi_budget;
+
+	#define RTL_NAPI_CONFIG(ndev, priv, function, weight)	netif_napi_add(ndev, &priv->napi, function, weight)
+	#define RTL_NAPI_QUOTA(budget, ndev)			min(budget, budget)
+	#define RTL_GET_PRIV(stuct_ptr, priv_struct)		container_of(stuct_ptr, priv_struct, stuct_ptr)
+	#define RTL_GET_NETDEV(priv_ptr)			struct net_device *dev = priv_ptr->dev;
+	#define RTL_RX_QUOTA(ndev, budget)			budget
+	#define RTL_NAPI_QUOTA_UPDATE(ndev, work_done, budget)
+ #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
+	#define RTL_NETIF_RX_COMPLETE(dev, napi)		netif_rx_complete(dev, napi)
+	#define RTL_NETIF_RX_SCHEDULE_PREP(dev, napi)		netif_rx_schedule_prep(dev, napi)
+	#define __RTL_NETIF_RX_SCHEDULE(dev, napi)		__netif_rx_schedule(dev, napi)
+ #endif
+ #if LINUX_VERSION_CODE == KERNEL_VERSION(2,6,29)
+	#define RTL_NETIF_RX_COMPLETE(dev, napi)		netif_rx_complete(napi)
+	#define RTL_NETIF_RX_SCHEDULE_PREP(dev, napi)		netif_rx_schedule_prep(napi)
+	#define __RTL_NETIF_RX_SCHEDULE(dev, napi)		__netif_rx_schedule(napi)
+ #endif
+ #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,29)
+	#define RTL_NETIF_RX_COMPLETE(dev, napi)		napi_complete(napi)
+	#define RTL_NETIF_RX_SCHEDULE_PREP(dev, napi)		napi_schedule_prep(napi)
+	#define __RTL_NETIF_RX_SCHEDULE(dev, napi)		__napi_schedule(napi)
+ #endif
+	#define RTL_NAPI_RETURN_VALUE work_done
+	#define RTL_NAPI_ENABLE(dev, napi)			napi_enable(napi)
+	#define RTL_NAPI_DISABLE(dev, napi)			napi_disable(napi)
+#endif	//LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
+
 /*****************************************************************************/
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
 #ifdef __CHECKER__
@@ -313,57 +444,6 @@ static inline int _kc_pci_dma_mapping_error(dma_addr_t dma_addr)
 #define ETHTOOL_OPS_COMPAT
 #endif /* 2.6.4 => 2.6.0 */
 
-/*****************************************************************************/
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-#undef ethtool_ops
-#define ethtool_ops _kc_ethtool_ops
-
-struct _kc_ethtool_ops {
-	int  (*get_settings)(struct net_device *, struct ethtool_cmd *);
-	int  (*set_settings)(struct net_device *, struct ethtool_cmd *);
-	void (*get_drvinfo)(struct net_device *, struct ethtool_drvinfo *);
-	int  (*get_regs_len)(struct net_device *);
-	void (*get_regs)(struct net_device *, struct ethtool_regs *, void *);
-	void (*get_wol)(struct net_device *, struct ethtool_wolinfo *);
-	int  (*set_wol)(struct net_device *, struct ethtool_wolinfo *);
-	u32  (*get_msglevel)(struct net_device *);
-	void (*set_msglevel)(struct net_device *, u32);
-	int  (*nway_reset)(struct net_device *);
-	u32  (*get_link)(struct net_device *);
-	int  (*get_eeprom_len)(struct net_device *);
-	int  (*get_eeprom)(struct net_device *, struct ethtool_eeprom *, u8 *);
-	int  (*set_eeprom)(struct net_device *, struct ethtool_eeprom *, u8 *);
-	int  (*get_coalesce)(struct net_device *, struct ethtool_coalesce *);
-	int  (*set_coalesce)(struct net_device *, struct ethtool_coalesce *);
-	void (*get_ringparam)(struct net_device *, struct ethtool_ringparam *);
-	int  (*set_ringparam)(struct net_device *, struct ethtool_ringparam *);
-	void (*get_pauseparam)(struct net_device *,
-	                       struct ethtool_pauseparam*);
-	int  (*set_pauseparam)(struct net_device *,
-	                       struct ethtool_pauseparam*);
-	u32  (*get_rx_csum)(struct net_device *);
-	int  (*set_rx_csum)(struct net_device *, u32);
-	u32  (*get_tx_csum)(struct net_device *);
-	int  (*set_tx_csum)(struct net_device *, u32);
-	u32  (*get_sg)(struct net_device *);
-	int  (*set_sg)(struct net_device *, u32);
-	u32  (*get_tso)(struct net_device *);
-	int  (*set_tso)(struct net_device *, u32);
-	int  (*self_test_count)(struct net_device *);
-	void (*self_test)(struct net_device *, struct ethtool_test *, u64 *);
-	void (*get_strings)(struct net_device *, u32 stringset, u8 *);
-	int  (*phys_id)(struct net_device *, u32);
-	int  (*get_stats_count)(struct net_device *);
-	void (*get_ethtool_stats)(struct net_device *, struct ethtool_stats *,
-	                          u64 *);
-} *ethtool_ops = NULL;
-
-#undef SET_ETHTOOL_OPS
-#define SET_ETHTOOL_OPS(netdev, ops) (ethtool_ops = (ops))
-
-#endif //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-
 /*****************************************************************************/
 /* Installations with ethtool version without eeprom, adapter id, or statistics
  * support */
@@ -684,49 +764,67 @@ struct _kc_ethtool_pauseparam {
 
 /*****************************************************************************/
 
+enum RTL8168_DSM_STATE {
+	DSM_MAC_INIT = 1,
+	DSM_NIC_GOTO_D3 = 2,
+	DSM_IF_DOWN = 3,
+	DSM_NIC_RESUME_D3 = 4,
+	DSM_IF_UP = 5,
+};
+
 enum RTL8168_registers {
-	MAC0 = 0,		/* Ethernet hardware address. */
-	MAC4 = 0x04,
-	MAR0 = 8,		/* Multicast filter. */
-	CounterAddrLow = 0x10,
-	CounterAddrHigh = 0x14,
-	TxDescStartAddrLow = 0x20,
-	TxDescStartAddrHigh = 0x24,
-	TxHDescStartAddrLow = 0x28,
-	TxHDescStartAddrHigh = 0x2c,
-	FLASH = 0x30,
-	ERSR = 0x36,
-	ChipCmd = 0x37,
-	TxPoll = 0x38,
-	IntrMask = 0x3C,
-	IntrStatus = 0x3E,
-	TxConfig = 0x40,
-	RxConfig = 0x44,
-	TCTR = 0x48,
-	Cfg9346 = 0x50,
-	Config0 = 0x51,
-	Config1 = 0x52,
-	Config2 = 0x53,
-	Config3 = 0x54,
-	Config4 = 0x55,
-	Config5 = 0x56,
-	TimeIntr = 0x58,
-	PHYAR = 0x60,
-	CSIDR = 0x64,
-	CSIAR = 0x68,
-	PHYstatus = 0x6C,
-	EPHYAR = 0x80,
-	DBG_reg = 0xD1,
-	RxMaxSize = 0xDA,
-	CPlusCmd = 0xE0,
-	IntrMitigate = 0xE2,
-	RxDescAddrLow = 0xE4,
-	RxDescAddrHigh = 0xE8,
-	EarlyTxThres = 0xEC,
-	FuncEvent = 0xF0,
-	FuncEventMask = 0xF4,
-	FuncPresetState = 0xF8,
-	FuncForceEvent = 0xFC,
+	MAC0			= 0x00,		/* Ethernet hardware address. */
+	MAC4			= 0x04,
+	MAR0			= 0x08,		/* Multicast filter. */
+	CounterAddrLow		= 0x10,
+	CounterAddrHigh		= 0x14,
+	CustomLED		= 0x18,
+	TxDescStartAddrLow	= 0x20,
+	TxDescStartAddrHigh	= 0x24,
+	TxHDescStartAddrLow	= 0x28,
+	TxHDescStartAddrHigh	= 0x2c,
+	FLASH			= 0x30,
+	ERSR			= 0x36,
+	ChipCmd			= 0x37,
+	TxPoll			= 0x38,
+	IntrMask		= 0x3C,
+	IntrStatus		= 0x3E,
+	TxConfig		= 0x40,
+	RxConfig		= 0x44,
+	TCTR			= 0x48,
+	Cfg9346			= 0x50,
+	Config0			= 0x51,
+	Config1			= 0x52,
+	Config2			= 0x53,
+	Config3			= 0x54,
+	Config4			= 0x55,
+	Config5			= 0x56,
+	TimeIntr		= 0x58,
+	PHYAR			= 0x60,
+	CSIDR			= 0x64,
+	CSIAR			= 0x68,
+	PHYstatus		= 0x6C,
+	MACDBG			= 0x6D,
+	GPIO			= 0x6E,
+	PMCH			= 0x6F,
+	ERIDR			= 0x70,
+	ERIAR			= 0x74,
+	EPHY_RXER_NUM		= 0x7C,
+	EPHYAR			= 0x80,
+	OCPDR			= 0xB0,
+	OCPAR			= 0xB4,
+	DBG_reg			= 0xD1,
+	RxMaxSize		= 0xDA,
+	EFUSEAR			= 0xDC,
+	CPlusCmd		= 0xE0,
+	IntrMitigate		= 0xE2,
+	RxDescAddrLow		= 0xE4,
+	RxDescAddrHigh		= 0xE8,
+	MTPS			= 0xEC,
+	FuncEvent		= 0xF0,
+	FuncEventMask		= 0xF4,
+	FuncPresetState		= 0xF8,
+	FuncForceEvent		= 0xFC,
 };
 
 enum RTL8168_register_content {
@@ -759,6 +857,12 @@ enum RTL8168_register_content {
 	/* Cfg9346Bits */
 	Cfg9346_Lock = 0x00,
 	Cfg9346_Unlock = 0xC0,
+	Cfg9346_EEDO = (1 << 0),
+	Cfg9346_EEDI = (1 << 1),
+	Cfg9346_EESK = (1 << 2),
+	Cfg9346_EECS = (1 << 3),
+	Cfg9346_EEM0 = (1 << 6),
+	Cfg9346_EEM1 = (1 << 7),
 
 	/* rx_mode_bits */
 	AcceptErr = 0x20,
@@ -774,8 +878,12 @@ enum RTL8168_register_content {
 	FSWInt = 0x01,
 
 	/* RxConfigBits */
-	RxCfgFIFOShift = 13,
+	Reserved2_shift = 13,
 	RxCfgDMAShift = 8,
+	RxCfg_128_int_en = (1 << 15),
+	RxCfg_fet_multi_en = (1 << 14),
+	RxCfg_half_refetch = (1 << 13),
+	RxCfg_9356SEL = (1 << 6),
 
 	/* TxConfigBits */
 	TxInterFrameGapShift = 24,
@@ -814,7 +922,7 @@ enum RTL8168_register_content {
 	EnableBist	= (1 << 15),
 	Macdbgo_oe	= (1 << 14),
 	Normal_mode	= (1 << 13),
-	Force_halpdup	= (1 << 12),
+	Force_halfdup	= (1 << 12),
 	Force_rxflow_en	= (1 << 11),
 	Force_txflow_en	= (1 << 10),
 	Cxpl_dbg_sel	= (1 << 9),//This bit is reserved in RTL8168B
@@ -827,7 +935,7 @@ enum RTL8168_register_content {
 	INTT_1		= 0x0001,
 	INTT_2		= 0x0002,
 	INTT_3		= 0x0003,
-	
+
 	/* rtl8168_PHYstatus */
 	TxFlowCtrl = 0x40,
 	RxFlowCtrl = 0x20,
@@ -861,13 +969,50 @@ enum RTL8168_register_content {
 	EPHYAR_Reg_shift = 16,
 	EPHYAR_Data_Mask = 0xffff,
 
-	/* CSI access */	
+	/* CSI access */
 	CSIAR_Flag = 0x80000000,
 	CSIAR_Write = 0x80000000,
 	CSIAR_Read = 0x00000000,
 	CSIAR_ByteEn = 0x0f,
 	CSIAR_ByteEn_shift = 12,
 	CSIAR_Addr_Mask = 0x0fff,
+
+	/* ERI access */
+	ERIAR_Flag = 0x80000000,
+	ERIAR_Write = 0x80000000,
+	ERIAR_Read = 0x00000000,
+	ERIAR_Addr_Align = 4, /* ERI access register address must be 4 byte alignment */
+	ERIAR_ExGMAC = 0,
+	ERIAR_MSIX = 1,
+	ERIAR_ASF = 2,
+	ERIAR_Type_shift = 16,
+	ERIAR_ByteEn = 0x0f,
+	ERIAR_ByteEn_shift = 12,
+
+	/* OCP GPHY access */
+	OCPDR_Write = 0x80000000,
+	OCPDR_Read = 0x00000000,
+	OCPDR_Reg_Mask = 0xFF,
+	OCPDR_Data_Mask = 0xFFFF,
+	OCPDR_GPHY_Reg_shift = 16,
+	OCPAR_Flag = 0x80000000,
+	OCPAR_GPHY_Write = 0x8000F060,
+	OCPAR_GPHY_Read = 0x0000F060,
+
+	/* E-FUSE access */
+	EFUSE_WRITE	= 0x80000000,
+	EFUSE_WRITE_OK	= 0x00000000,
+	EFUSE_READ	= 0x00000000,
+	EFUSE_READ_OK	= 0x80000000,
+	EFUSE_Reg_Mask	= 0x03FF,
+	EFUSE_Reg_Shift	= 8,
+	EFUSE_Check_Cnt	= 300,
+	EFUSE_READ_FAIL	= 0xFF,
+	EFUSE_Data_Mask	= 0x000000FF,
+
+	/* GPIO */
+	GPIO_en = (1 << 0),
+
 };
 
 enum _DescStatusBit {
@@ -880,7 +1025,7 @@ enum _DescStatusBit {
 	/*------ offset 0 of tx descriptor ------*/
 	LargeSend	= (1 << 27), /* TCP Large Send Offload (TSO) */
 	MSSShift	= 16,        /* MSS value position */
-	MSSMask		= 0xfff,     /* MSS value + LargeSend bit: 12 bits */
+	MSSMask		= 0x7ffU,    /* MSS value + LargeSend bit: 12 bits */
 	TxIPCS		= (1 << 18), /* Calculate IP checksum */
 	TxUDPCS		= (1 << 17), /* Calculate UDP/IP checksum */
 	TxTCPCS		= (1 << 16), /* Calculate TCP/IP checksum */
@@ -919,6 +1064,55 @@ enum _DescStatusBit {
 	/*@@@@@@ offset 4 of rx descriptor => bits for RTL8168C/CP only		end @@@@@@*/
 };
 
+enum features {
+//	RTL_FEATURE_WOL	= (1 << 0),
+	RTL_FEATURE_MSI	= (1 << 1),
+};
+
+enum wol_capability {
+	WOL_DISABLED = 0,
+	WOL_ENABLED = 1
+};
+
+enum bits {
+	BIT_0 = (1 << 0),
+	BIT_1 = (1 << 1),
+	BIT_2 = (1 << 2),
+	BIT_3 = (1 << 3),
+	BIT_4 = (1 << 4),
+	BIT_5 = (1 << 5),
+	BIT_6 = (1 << 6),
+	BIT_7 = (1 << 7),
+	BIT_8 = (1 << 8),
+	BIT_9 = (1 << 9),
+	BIT_10 = (1 << 10),
+	BIT_11 = (1 << 11),
+	BIT_12 = (1 << 12),
+	BIT_13 = (1 << 13),
+	BIT_14 = (1 << 14),
+	BIT_15 = (1 << 15),
+	BIT_16 = (1 << 16),
+	BIT_17 = (1 << 17),
+	BIT_18 = (1 << 18),
+	BIT_19 = (1 << 19),
+	BIT_20 = (1 << 20),
+	BIT_21 = (1 << 21),
+	BIT_22 = (1 << 22),
+	BIT_23 = (1 << 23),
+	BIT_24 = (1 << 24),
+	BIT_25 = (1 << 25),
+	BIT_26 = (1 << 26),
+	BIT_27 = (1 << 27),
+	BIT_28 = (1 << 28),
+	BIT_29 = (1 << 29),
+	BIT_30 = (1 << 30),
+	BIT_31 = (1 << 31)
+};
+
+enum effuse {
+	EFUSE_SUPPORT = 1,
+	EFUSE_NOT_SUPPORT = 0,
+};
 #define RsvdMask	0x3fffc000
 
 struct TxDesc {
@@ -957,6 +1151,11 @@ struct rtl8168_private {
 	void __iomem *mmio_addr;	/* memory map physical address */
 	struct pci_dev *pci_dev;	/* Index of PCI device */
 	struct net_device *dev;
+#ifdef CONFIG_R8168_NAPI
+   #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+	struct napi_struct napi;
+   #endif
+#endif
 	struct net_device_stats stats;	/* statistics of net device */
 	spinlock_t lock;		/* spin lock flag */
 	spinlock_t phy_lock;		/* spin lock flag for GPHY */
@@ -966,7 +1165,7 @@ struct rtl8168_private {
 	u32 tx_ip_csum_cmd;
 	int max_jumbo_frame_size;
 	int chipset;
-	int mcfg;
+	u32 mcfg;
 	u32 cur_rx; /* Index into the Rx descriptor buffer of next Rx pkt. */
 	u32 cur_tx; /* Index into the Tx descriptor buffer of next Rx pkt. */
 	u32 dirty_rx;
@@ -980,31 +1179,28 @@ struct rtl8168_private {
 	unsigned rx_buf_sz;
 	int rx_fifo_overflow;
 	struct timer_list esd_timer;
+	struct timer_list link_timer;
+	int old_link_status;
 	struct pci_resource pci_cfg_space;
 	unsigned int esd_flag;
 	unsigned int pci_cfg_is_read;
+	unsigned int rtl8168_rx_config;
 	u16 cp_cmd;
 	u16 intr_mask;
 	int phy_auto_nego_reg;
 	int phy_1000_ctrl_reg;
-	u8 mac_addr[NODE_ADDRESS_SIZE];
-#ifdef	RTL_IOCTL_SUPPORT
-	int loopback_tx_schedule;
-	int loopback_state;
-	int loopback_type;
-	struct tasklet_struct loopback_tasklet;
-	struct sk_buff *loopback_skb;
-	int loopback_tx_ok;
-	int loopback_rx_ok;
-	int loopback_tx_err;
-	int loopback_rx_err;
-	unsigned char *loopback_pkt_payload;
-	int loopback_pkt_payload_len;
-#endif	//RTL_IOCTL_SUPPORT
-
+	u8 org_mac_addr[NODE_ADDRESS_SIZE];
 #ifdef CONFIG_R8168_VLAN
 	struct vlan_group *vlgrp;
 #endif
+	u8	wol_enabled;
+	u8	efuse;
+	u8	eeprom_type;
+	u8	autoneg;
+	u8	duplex;
+	u16	speed;
+	u16	eeprom_len;
+
 	int (*set_speed)(struct net_device *, u8 autoneg, u16 speed, u8 duplex);
 	void (*get_settings)(struct net_device *, struct ethtool_cmd *);
 	void (*phy_reset_enable)(struct net_device *);
@@ -1015,6 +1211,61 @@ struct rtl8168_private {
 #else
 	struct delayed_work task;
 #endif
-	unsigned wol_enabled : 1;
+	unsigned features;
 };
 
+enum eetype {
+	EEPROM_TYPE_NONE=0,
+	EEPROM_TYPE_93C46,
+	EEPROM_TYPE_93C56,
+	EEPROM_TWSI
+};
+
+enum mcfg {
+	CFG_METHOD_1=0,
+	CFG_METHOD_2,
+	CFG_METHOD_3,
+	CFG_METHOD_4,
+	CFG_METHOD_5,
+	CFG_METHOD_6,
+	CFG_METHOD_7,
+	CFG_METHOD_8,
+	CFG_METHOD_9 ,
+	CFG_METHOD_10,
+	CFG_METHOD_11,
+	CFG_METHOD_12,
+	CFG_METHOD_13,
+	CFG_METHOD_14,
+	CFG_METHOD_15,
+	CFG_METHOD_16,
+	CFG_METHOD_17,
+	CFG_METHOD_18,
+	CFG_METHOD_19,
+	CFG_METHOD_20,
+	CFG_METHOD_MAX,
+	CFG_METHOD_DEFAULT = 0xFF
+};
+
+#define OOB_CMD_RESET		0x00
+#define OOB_CMD_DRIVER_START	0x05
+#define OOB_CMD_DRIVER_STOP	0x06
+#define OOB_CMD_SET_IPMAC	0x41
+
+extern void mdio_write(struct rtl8168_private *tp, u32 RegAddr, u32 value);
+extern void rtl8168_ephy_write(void __iomem *ioaddr, int RegAddr, int value);
+extern void OCP_write(struct rtl8168_private *tp, u8 mask, u16 Reg, u32 data);
+extern void OOB_notify(struct rtl8168_private *tp, u8 cmd);
+extern void rtl8168_init_ring_indexes(struct rtl8168_private *tp);
+extern int rtl8168_eri_write(void __iomem *ioaddr, int addr, int len, u32 value, int type);
+extern u32 mdio_read(struct rtl8168_private *tp, u32 RegAddr);
+extern u32 OCP_read(struct rtl8168_private *tp, u8 mask, u16 Reg);
+extern u32 rtl8168_eri_read(void __iomem *ioaddr, int addr, int len, int type);
+extern u16 rtl8168_ephy_read(void __iomem *ioaddr, int RegAddr);
+
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
+#define netdev_mc_count(dev) ((dev)->mc_count)
+#define netdev_mc_empty(dev) (netdev_mc_count(dev) == 0)
+#define netdev_for_each_mc_addr(mclist, dev) \
+	for (mclist = dev->mc_list; mclist; mclist = mclist->next)
+#endif
